/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import lattice.Ray;
import math.VecMath;

class AnimateRotation<T extends Number> implements Runnable {

	public static final Boolean INTERPOLATE = true;
	private List<T[]> sincoskdphi;
	private List<T> kdphi;
	final Controller<T> ctrl;
	protected int pause;
	protected List<String> descr;
	protected boolean stopOnEmptyCache;
	protected boolean skipIfOverloaded;
	protected List<Ray<T>> rayCache;
	protected boolean run;

	public AnimateRotation(Controller<T> c) {
		ctrl = c;
		run = false;
		stopOnEmptyCache = false;
		skipIfOverloaded = true;
		rayCache = new LinkedList<Ray<T>>();
		descr = new LinkedList<String>();
		sincoskdphi = new ArrayList<T[]>();
		kdphi = new ArrayList<T>();
	}
	public synchronized void run() {
		Model<T> model = ctrl.getModel();
		model.updateWhileCalc(false);
		boolean interpolate = false;
		while(run) {
			ctrl.stopCalculation();
			int sleep = pause;
			int qs = ctrl.getView().getReflectionVisualizer().getQueueSize();
			int steps = 2;
			if(skipIfOverloaded)
				steps = (int)(Math.pow(qs,2)/50+2);
			else if(interpolate||(rayCache.size()>0&&
					rayCache.get(0).getAttribute()==INTERPOLATE)){
				steps = 1;
				interpolate = !interpolate;		
			}
			sleep *= steps;
			while(kdphi.size()<steps){
				T[] sckdphi = sincoskdphi.get(sincoskdphi.size()-1); 
				T[] scdphi = sincoskdphi.get(0);
				sincoskdphi.add(sinCosKPlusOneDphi(sckdphi,scdphi));
				T dphik = kdphi.get(kdphi.size()-1);
				T dphi = kdphi.get(0);
				kdphi.add(model.getMath().add(dphi,dphik));
			}
			ctrl.waitForCalculatorRelease();
			if(rayCache.size()>0){
				String s = descr.size()>0?descr.remove(0):null;
				model.setRay(rayCache.remove(0),s,Model.DYNAMIC_CALC);
			} else 
				model.setCalcMode(Model.DYNAMIC_CALC);
			model.rotateGrid(kdphi.get(steps-1),sincoskdphi.get(steps-1));
			if(stopOnEmptyCache&&(rayCache.size()==0))
				model.updateWhileCalc(true);
			ctrl.startCalculation();
			if(stopOnEmptyCache&&(rayCache.size()==0)){
				ctrl.waitForCalculatorRelease();
				run = false;
			} else
				try {
					Thread.sleep(sleep);
				} catch(InterruptedException e) {}
		}
		rayCache.clear();
		descr.clear();
		sincoskdphi.clear();
		kdphi.clear();
		model.updateWhileCalc(true);
	}
	private T[] sinCosKPlusOneDphi(T[] sckdphi, T[] scdphi){
		VecMath<T> mt = ctrl.getModel().getMath();
		T[] retVal = mt.util.array(2);
		retVal[0] = mt.add(mt.mul(sckdphi[0],scdphi[1]),mt.mul(sckdphi[1],scdphi[0]));
		retVal[1] = mt.sub(mt.mul(sckdphi[1],scdphi[1]),mt.mul(sckdphi[0],scdphi[0]));
		return retVal;
	}
	public void setPhi(float phi) {
		VecMath<T> math = ctrl.getModel().getMath();
		kdphi.add(math.util.copy(math.util.cast(phi/2),math.PRECISION));
		sincoskdphi.add(math.sincos(kdphi.get(0)));
	}
	public void setPhi(T phi) {
		VecMath<T> math = ctrl.getModel().getMath();
		phi = math.div(phi,math.TWO);
		kdphi.add(math.util.copy(phi,math.PRECISION));
		sincoskdphi.add(math.sincos(kdphi.get(0)));
	}
	public float getPhi(){
		if(kdphi.size()>1)
			return kdphi.get(1).floatValue();
		return 0;
	}
}
