/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

import javax.swing.ImageIcon;

import lattice.Box2D;
import lattice.Circle;
import lattice.IShape;
import lattice.Ray;
import lattice.RayPolygon;
import lattice.ReflectionEngine;
import lattice.ShapeContainer;
import math.Util;
import math.VecMath;
import pattern.BaseLightPattern;
import pattern.LightChar;

import common.Log;

public class LightCharMap<T extends Number>  {

	static final byte ICON_INI_SIZE = 25;
	private static final byte SHAPE_SIZE = 40;
	private static HashMap<String,float[]> polyMap;
	private boolean calculate = true;
	private final VecMath<T> math;
	private final VecMath<Float> fmath;
	final HashMap<String,ImageIcon> iconMap;
	final LightChar[] lChars;
	int iconSize;
	protected Component refreshComp;
	byte finished = 0;

	public LightCharMap(LightChar[] lcs,VecMath<T> mt,VecMath<Float> fmt){
		this.math = mt;
		this.fmath = fmt;
		this.lChars = lcs;
		Arrays.sort(lChars);
		this.iconMap = new HashMap<String,ImageIcon>();
		this.iconSize = ICON_INI_SIZE;
		if(polyMap==null){
			Log.getIstc().enableLogging(false);
			polyMap = new HashMap<String,float[]>();
			Thread t1 = new Thread(new Runnable() {
				public void run() {
					for(int k = 0;k<lChars.length&&calculate;k+=2)
						polyMap.put(lChars[k].getChar(),calcPolygon(lChars[k]));
					refresh();
					if(++finished==2)
						Log.getIstc().enableLogging(true);
				}
			});
			Thread t2 = new Thread(new Runnable() {
				public void run() {
					for(int k = lChars.length-1;k>0&&calculate;k-=2) {
						polyMap.put(lChars[k].getChar(),calcPolygon(lChars[k]));
						refresh();
					}
					if(++finished==2)
						Log.getIstc().enableLogging(true);					
				}
			});
			t1.start();
			t2.start();
		}
	}
	private void refresh(){
		if(refreshComp!=null)
			refreshComp.repaint();
	}
	void buildIcons(int size) {
		float gap2 = ICON_INI_SIZE*1f;
		Float[] corner2 = new Float[]{-gap2,-gap2};
		Box2D<Float> box = new Box2D<Float>(corner2,gap2*2,fmath);
		PixelMapFloat pixMap = new PixelMapFloat(size,size,(short)0,(short)0,box);
		for(int k = 0;k<lChars.length&&calculate;k++) {
			LightChar lc = lChars[k];
			if(polyMap.get(lc.getChar())==null)
				continue;
			ImageIcon ic = iconMap.get(lc.getChar());
			if(ic!=null&&ic.getIconWidth()==size)
				continue;
			BufferedImage bimg;
			bimg = new BufferedImage(size,size,BufferedImage.TYPE_INT_RGB);
			Graphics2D ig = bimg.createGraphics();
			ig.setColor(Color.black);
			ig.fillRect(0,0,size,size);
			ig.setColor(Color.white);
			ig.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
								RenderingHints.VALUE_ANTIALIAS_ON);
			float[] polygon = polyMap.get(lc.getChar());
			for(int i = 0;i<polygon.length-2;i+=2) {
				short[] p1 = new short[2];
				short[] p2 = new short[2];
				Float[] pt = new Float[]{polygon[i],polygon[i+1]};
				pixMap.world2Pix(pt,p1);
				pt = new Float[]{polygon[i+2],polygon[i+3]};
				pixMap.world2Pix(pt,p2);
				ig.drawLine(p1[0],p1[1],p2[0],p2[1]);
			}
			ig.dispose();
			bimg.flush();
			ImageIcon icon = new ImageIcon(bimg);
			iconMap.put(lc.getChar(),icon);
		}
	}
	private float[] calcPolygon(LightChar lc){
		BaseLightPattern<T> blp = new BaseLightPattern<T>(math);
		blp.setMaxShapeSize(math.HUNDRED);
		blp.setGridSize((short)2);
		blp.setPropagating(true);
		blp.setLogging(false);
		List<short[]> cells = lc.getCellSequence();
		blp.setCellSequence(new ArrayList<short[]>(cells));
		blp.setStartIndex(cells.size());
		blp.setInitialGradients(new ArrayList<Float>(lc.getInitialGradients()));
		Util<T> ut = math.util;
		if(lc.getOrientation()==LightChar.VERTICAL) {
			if(cells.get(0)[0]==1) {
				blp.getInitialGradients().set(0,-0.2f);
				blp.setStartBndVal(ut.copy(ut.cast(-0.3),math.PRECISION));
				blp.setStartBndValType(BaseLightPattern.GRAD_RIGHT);
			} else {
				blp.getInitialGradients().set(0,0.2f);
				blp.setStartBndVal(ut.copy(ut.cast(0.3),math.PRECISION));
				blp.setStartBndValType(BaseLightPattern.GRAD_LEFT);
			}
			if(cells.get(cells.size()-1)[0]==1) {
				blp.getInitialGradients().set(cells.size()-1,0.2f);
				blp.setEndBndVal(ut.copy(ut.cast(0.3),math.PRECISION));
				blp.setEndBndValType(BaseLightPattern.GRAD_LEFT);
			} else {
				blp.getInitialGradients().set(cells.size()-1,-0.2f);
				blp.setEndBndVal(ut.copy(ut.cast(-0.3),math.PRECISION));
				blp.setEndBndValType(BaseLightPattern.GRAD_RIGHT);
			}
		} else {
			if(cells.get(0)[1]==0)
				blp.setStartBndVal(ut.copy(math.MINUS_ONE,math.PRECISION));
			else
				blp.setStartBndVal(ut.copy(math.ONE,math.PRECISION));
			if(cells.get(cells.size()-1)[1]==0)
				blp.setEndBndVal(ut.copy(math.ONE,math.PRECISION));
			else
				blp.setEndBndVal(ut.copy(math.MINUS_ONE,math.PRECISION));
			blp.setStartBndValType(BaseLightPattern.GRAD_RIGHT);
			blp.setEndBndValType(BaseLightPattern.GRAD_RIGHT);
		}
		T gap = ut.cast(ICON_INI_SIZE);
		T r = math.mul(math.div(gap,ut.cast(200)),SHAPE_SIZE);
		IShape<T> shape = new Circle<T>(math.ZERO2D.clone(),r,math);
		T[] corner = ut.array(2);
		corner[0] = math.neg(gap);
		corner[1] = math.neg(gap);
		blp.setParameters(corner.clone(),gap,r);
		T phi = blp.getRotation();
		phi = math.util.copy(phi,blp.getAccuracy());
		T[] sinCos = math.sincos(phi);
		T[] start = blp.getStart(gap,phi,sinCos);
		T[] dir = ut.array(2);
		dir[0] = math.MINUS_ONE;
		dir[1] = math.ZERO;
		Ray<T> ray = new Ray<T>(start,dir,ut);
		ShapeContainer<T> shapeCtr;
		shapeCtr = new ShapeContainer<T>(corner.clone(),(short)3,gap,math);
		shapeCtr.setShape(shape);
		shapeCtr.rotate(math.ZERO2D,null,sinCos[0],sinCos[1]);
		RayPolygon<T> rayPoly = new RayPolygon<T>(ut,8);
		ReflectionEngine<T> engine = new ReflectionEngine<T>(math);
		engine.calcStatic(ray,shapeCtr,rayPoly,null);
		if(lc.getOrientation()!=LightChar.VERTICAL) {
			T[] st = rayPoly.get(0).clone();
			math.scaleAdd(st,math.neg(math.FIVE),rayPoly.getInRay().getDir(),st);
			rayPoly.add(0,st);
			T[] end = rayPoly.get(rayPoly.size()-1).clone();
			math.scaleAdd(end,math.FIVE,rayPoly.getOutRay().getDir(),end);
			rayPoly.add(end);
		}
		float[] poly = new float[2*rayPoly.size()];
		for(int i = 0;i<rayPoly.size();i++){
			math.rotate2D(rayPoly.get(i),math.ZERO2D,math.neg(sinCos[0]),sinCos[1]);
			poly[2*i] = rayPoly.get(i)[0].floatValue();
			poly[2*i+1] = rayPoly.get(i)[1].floatValue();
		}
		return poly;
	}
	public void dispose() {
		calculate = false;
	}
}