/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import lattice.Box2D;
import math.VecMath;

public final class PixelMap<T extends Number> extends PixelMapAbstract<T> {

	private final VecMath<T> math;
	private T invBoxWidth;
	private T invBoxHeight;
	private T invCanvHeight;
	private T invCanvWidth;
	public PixelMap(int w,int h,short bx,short by,Box2D<T> world,VecMath<T> math) {
		super(w,h,bx,by,world);
		this.math = math;
		invCanv();
		this.invBoxWidth = math.inv(box.getWidth());
		this.invBoxHeight = math.inv(box.getHeight());
	}
	private void invCanv(){
		short cw = canvasWidth();
		short ch = canvasHeight();
		this.invCanvWidth = cw > 0 ? math.inv(math.util.cast(cw)) : math.ONE;
		this.invCanvHeight = ch > 0 ? math.inv(math.util.cast(ch)) : math.ONE;
	}
	// Transforms x world coordinate to canvas (screen without border) coordinate.
	@Override
	public short xWorld2Pix(T x) {
		T x1 = math.mul(invBoxWidth,canvasWidth());
		T x2 = math.sub(x,box.getCorner()[0]);
		return math.mul(x1,x2).shortValue();
	}
	// Transforms y world coordinate to canvas (screen without border) coordinate.
	@Override
	public short yWorld2Pix(T y) {
		T y1 = math.sub(math.ONE,math.mul(math.sub(y,box.getCorner()[1]),invBoxHeight));
		return math.mul(y1,canvasHeight()).shortValue();
	}
	@Override
	public T xPix2World(short xpix) {
		T t1 = math.mul(box.getWidth(),xpix);
		return math.add(math.mul(t1,invCanvWidth),box.getCorner()[0]);
	}
	@Override
	public T yPix2World(short ypix) {
		T t1 = math.add(box.getHeight(),box.getCorner()[1]);
		T t2 = math.mul(box.getHeight(),ypix);
		return math.sub(t1,math.mul(t2,invCanvHeight));
	}
	@Override
	public short xLengthWorld2Pix(T x) {
		return (short)Math.abs(xWorld2Pix(math.ZERO)-xWorld2Pix(x));
	}
	@Override
	public short yLengthWorld2Pix(T y) {
		return (short)Math.abs(yWorld2Pix(math.ZERO)-yWorld2Pix(y));
	}
	@Override
	public T xLengthPix2World(short x) {
		return math.abs(math.sub(xPix2World((short)0),xPix2World(x)));
	}
	@Override
	public T yLengthPix2World(short y) {
		return math.abs(math.sub(yPix2World((short)0),yPix2World(y)));
	}
	@Override
	public void setSreenSize(short width,short height) {
		if(this.width!=width||this.height!=height) {
			this.width = width;
			this.height = height;
			invCanv();
		}
	}
	@Override
	public void worldChanged(Box2D<T> newBox) {
		super.worldChanged(newBox);
		this.invBoxWidth = math.inv(box.getWidth());
		this.invBoxHeight = math.inv(box.getHeight());
	}
	@Override
	public void setBorder(short bx,short by) {
		super.setBorder(bx,by);
		invCanv();
	}
}
