/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import java.awt.Rectangle;

import lattice.Box2D;
import lattice.IBox;

public abstract class PixelMapAbstract<T extends Number> {

	private short borderX;
	private short borderY;
	protected int width;
	protected int height;
	protected final Box2D<T> box;
	public PixelMapAbstract(int w,int h,short bx,short by,Box2D<T> worldBox) {
		this.borderX = bx;
		this.borderY = by;
		this.width = w;
		this.height = h;
		this.box = worldBox;
	}
	/**
	 * Transforms x world coordinate to canvas (screen without border) coordinate.
	 * @param x
	 * @return
	 */
	public abstract short xWorld2Pix(T x);
	/**
	 * Transforms y world coordinate to canvas (screen without border) coordinate.
	 * @param y
	 * @return
	 */
	public abstract short yWorld2Pix(T y);
	/**
	 * Transforms world coordinates to canvas (screen without border) coordinates.
	 * @param xy
	 * @return
	 */
	public final void world2Pix(T[] xy,short[] ret) {
		ret[0] = xWorld2Pix(xy[0]);
		ret[1] = yWorld2Pix(xy[1]);
	}
	public abstract T xPix2World(short xpix);
	public abstract T yPix2World(short ypix);
	public final void pix2World(short xpix,short ypix,T[] ret) {
		ret[0] = xPix2World(xpix);
		ret[1] = yPix2World(ypix);
	}
	public final void pix2World(short[] pix,T[] ret) {
		ret[0] = xPix2World(pix[0]);
		ret[1] = yPix2World(pix[1]);
	}
	public abstract short xLengthWorld2Pix(T x);
	public abstract short yLengthWorld2Pix(T y);
	public abstract T xLengthPix2World(short x);
	public abstract T yLengthPix2World(short y);
	public final Rectangle box2Pix(Box2D<T> b) {
		short[] corner = new short[2];
		world2Pix(b.getCorner(),corner);
		toScreen(corner);
		short[] lengths = new short[2];
		lengths[0] = xLengthWorld2Pix(b.getLengths()[0]);
		lengths[1] = yLengthWorld2Pix(b.getLengths()[1]);
		return new Rectangle(corner[0],corner[1]-lengths[1],lengths[0],lengths[1]);
	}
	public final short toCanvasX(short s) {
		return (short)(s-borderX);
	}
	public final short toCanvasY(short s) {
		return (short)(s-borderY);
	}
	public final short[] toCanvas(short x,short y) {
		return new short[]{toCanvasX(x),toCanvasY(y)};
	}
	public final void clip2Canvas(short[] xy,short margin) {
		xy[0] = xy[0]<margin?margin:xy[0];
		xy[0] = (short) (xy[0]>canvasWidth()-margin?canvasWidth()-margin:xy[0]);
		xy[1] = xy[1]<margin?margin:xy[1];
		xy[1] = (short) (xy[1]>canvasHeight()-margin?canvasHeight()-margin:xy[1]);
	}
	/**
	 * Transforms canvas (screen without border) pixel coordinate to screen pixel coordinate.
	 * @param s
	 * @return
	 */
	public final short toScreenX(short s) {
		return (short)(s+borderX);
	}
	public final short toScreenY(short s) {
		return (short)(s+borderY);
	}

	/**
	 * Transforms canvas (screen without border) pixel coordinates to screen pixel coordinates.
	 * @param s
	 * @return
	 */
	public final void toScreen(short[] s) {
		s[0] = toScreenX(s[0]);
		s[1] = toScreenY(s[1]);
	}
	public final short canvasWidth() {
		return (short)(width-2*borderX);
	}
	public final short canvasHeight() {
		return (short)(height-2*borderY);
	}
	public abstract void setSreenSize(short width,short height);
	public void worldChanged(Box2D<T> newWorld) {
		box.setCorner(newWorld.getCorner());
		box.setLengths(newWorld.getLengths());
	}
	public final IBox<T> getWorldBox() {
		return box;
	}
	public void setBorder(short bx,short by) {
		borderX = bx;
		borderY = by;
	}
}
