/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc;
import lattice.Box2D;

/**
 * The PixelMapFloat implements the PixelMapAbstract<Float> with
 * primitive types whenever possible to maximize speed and minimize
 * creation of primitive wrappers.
 * @author zoltan
 */
public final class PixelMapFloat extends PixelMapAbstract<Float> {

	// Local variables for transformation speed up
	private float worldWidth;
	private float worldHeight;
	private float[] worldCorner;
	private float invWorldWidth;
	private float invWorldHeight;
	private float invCanvWidth;
	private float invCanvHeight;
	public PixelMapFloat(int w,int h,short bx,short by,Box2D<Float> world) {
		super(w,h,bx,by,world);
		invCanv();
		updateWorldVariables();
	}
	private void invCanv(){
		short cw = canvasWidth();
		short ch = canvasHeight();
		this.invCanvWidth = cw > 0 ? 1f/cw : 1f;
		this.invCanvHeight = ch > 0 ? 1f/ch : 1f;		
	}
	private void updateWorldVariables() {
		worldWidth = box.getLengths()[0];
		worldHeight = box.getLengths()[1];
		invWorldWidth = 1/worldWidth;
		invWorldHeight = 1/worldHeight;
		worldCorner = new float[2];
		worldCorner[0] = box.getCorner()[0];
		worldCorner[1] = box.getCorner()[1];
	}
	@Override
	public short xWorld2Pix(Float x) {
		return (short)Math.round((x-worldCorner[0])*invWorldWidth*canvasWidth());
	}
	@Override
	public short yWorld2Pix(Float y) {
		float y1 = 1-(y-worldCorner[1])*invWorldHeight;
		return (short)Math.round(y1*canvasHeight());
	}
	@Override
	public short xLengthWorld2Pix(Float x) {
		return (short)Math.round(Math.abs(xWorld2Pix(0f)-xWorld2Pix(x)));
	}
	@Override
	public short yLengthWorld2Pix(Float y) {
		return (short)Math.round(Math.abs(yWorld2Pix(0f)-yWorld2Pix(y)));
	}
	@Override
	public Float xPix2World(short xpix) {
		float t1 = worldWidth*xpix;
		return t1*invCanvWidth+worldCorner[0];
	}
	@Override
	public Float yPix2World(short ypix) {
		float t1 = worldHeight+worldCorner[1];
		float t2 = worldHeight*ypix;
		return t1-t2*invCanvHeight;
	}
	@Override
	public void setSreenSize(short width,short height) {
		if(this.width!=width||this.height!=height) {
			this.width = width;
			this.height = height;
			invCanv();
		}
	}
	@Override
	public void worldChanged(Box2D<Float> newBox) {
		super.worldChanged(newBox);
		updateWorldVariables();
	}
	@Override
	public Float xLengthPix2World(short x) {
		return Math.abs(xPix2World((short)0)-xPix2World(x));
	}
	@Override
	public Float yLengthPix2World(short y) {
		return Math.abs(yPix2World((short)0)-yPix2World(y));
	}
	@Override
	public void setBorder(short bx,short by) {
		super.setBorder(bx,by);
		invCanv();
	}
}
