/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package mvc.mgr;
import java.util.ArrayList;

import javax.swing.SwingUtilities;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

public class SeqenceTableModel implements TableModel {

	private final ArrayList<TableModelListener> listeners;
	private final String[] columnNames;
	private final ArrayList<Row> rows;
	private ArrayList<Row> rowBuffer;
	public SeqenceTableModel(String[] columnNames) {
		this.listeners = new ArrayList<TableModelListener>();
		this.columnNames = columnNames;
		this.rows = new ArrayList<Row>();
		for(int i = 0;i<20;i++)
			rows.add(new Row());
	}
	public void updateTable() {
		SwingUtilities.invokeLater(new Thread() {

			public void run() {
				for(TableModelListener l:listeners)
					l.tableChanged(new TableModelEvent(SeqenceTableModel.this));
			}
		});
	}
	public int getRowCount() {
		return rows.size();
	}
	public int getColumnCount() {
		return columnNames.length;
	}
	public String getColumnName(int columnIndex) {
		if(columnIndex<columnNames.length)
			return columnNames[columnIndex];
		return null;
	}
	public Class<?> getColumnClass(int cIdx) {
		switch(cIdx) {
			case 0:
				return String.class;
			case 1:
				return String.class;
			case 2:
				return String.class;
		}
		return null;
	}
	public boolean isCellEditable(int rIdx,int cIdx) {
		switch(cIdx) {
			case 0:
				return false;
			case 1:
				return true;
			case 2:
				return true;
		}
		return false;
	}
	public Object getValueAt(int rIdx,int cIdx) {
		switch(cIdx) {
			case 0:
				return (rIdx+1)+"";
			case 1:
				return rows.get(rIdx).cell;
			case 2:
				return rows.get(rIdx).value;
		}
		return null;
	}
	public void setValueAt(Object val,int rIdx,int cIdx) {
		switch(cIdx) {
			case 1:
				rows.get(rIdx).cell = (String)val;
				break;
			case 2:
				rows.get(rIdx).value = (String)val;
				break;
		}
	}
	public void addTableModelListener(TableModelListener l) {
		listeners.add(l);
	}
	public void removeTableModelListener(TableModelListener l) {
		listeners.remove(l);
	}
	public void clear() {
		for(Row r:rows)
			r.clear();
		updateTable();
	}
	public void removeRows(int[] rowPos) {
		ArrayList<Row> delRows = new ArrayList<Row>();
		for(int r:rowPos)
			delRows.add(rows.get(r));
		for(Row r:delRows)
			rows.remove(r);
		updateTable();
	}
	public void insertRow(int row) {
		if(rows.size()>=row) {
			rows.add(row,new Row());
			updateTable();
		}
	}
	public void copyBuffer(int pos) {
		if(rowBuffer!=null)
			for(int i = rowBuffer.size()-1;i>=0;i--)
				rows.add(pos,rowBuffer.get(i));
		updateTable();
	}
	public void bufferRows(int[] rowIndexes) {
		if(rowIndexes!=null)
			rowBuffer = new ArrayList<Row>();
		for(int i:rowIndexes)
			rowBuffer.add(new Row(rows.get(i)));
	}
	public int getLastRowPos() {
		for(int i = 0;i<rows.size();i++) {
			String c = rows.get(i).cell;
			if(c==null||c.trim().compareTo("")==0)
				return i-1;
		}
		return -1;
	}
	public void incX(int[] rowPos) {
		alter(rowPos,true,"(",",");
	}
	public void decX(int[] rowPos) {
		alter(rowPos,false,"(",",");
	}
	public void incY(int[] rowPos) {
		alter(rowPos,true,",",")");
	}
	public void decY(int[] rowPos) {
		alter(rowPos,false,",",")");
	}
	private void alter(int[] rowPos,boolean b,String s1,String s2) {
		for(int pos:rowPos) {
			Row row = rows.get(pos);
			int i1 = row.cell.indexOf(s1);
			int i2 = row.cell.indexOf(s2);
			if(i1!=-1&&i2!=-1) {
				String valstr = row.cell.substring(i1+1,i2);
				short val = Short.parseShort(valstr);
				val += b?+1:-1;
				String c = row.cell;
				row.cell = c.substring(0,i1+1)+val+c.substring(i2);
			}
		}
		updateTable();
	}
	private class Row {

		private String cell;
		private String value;
		Row() {}
		Row(Row r) {
			this.cell = r.cell;
			this.value = r.value;
		}
		void clear() {
			cell = null;
			value = null;
		}
	}
}