/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package parser;
import java.util.LinkedList;

import math.VecMath;

public class ExpressionTreeBuilder<T extends Number> extends Parser {

	private final LinkedList<Expression<T>> expressions;
	private final VecMath<T> math;
	public ExpressionTreeBuilder(VecMath<T> mt) {
		this.math = mt;
		this.expressions = new LinkedList<Expression<T>>();
	}
	public ExpressionTree<T> build(String mathExpr) throws Exception {
		Expression<T> root = new OperatorExpression<T>(math);
		ExpressionTree<T> tree = new ExpressionTree<T>(root);
		expressions.clear();
		expressions.add(root);
		parse(mathExpr);
		return tree;
	}
	@Override
	void closeBrace() {
		expressions.removeLast();
		Expression<T> expr = expressions.getLast();
		if(expr instanceof FunctionExpression)
			expressions.removeLast();
	}
	@Override
	void openBrace() {
		OperatorExpression<T> op = new OperatorExpression<T>(math);
		Expression<T> expr = expressions.getLast();
		expr.addChild(op);
		expressions.add(op);
	}
	@Override
	void operator(String op) throws Exception {
		Expression<T> expr = expressions.getLast();
		if(expr instanceof OperatorExpression)
			((OperatorExpression<T>)expr).addOperator(op);
		else
			throw new Exception("Bad expression at operator "+op);
	}
	@Override
	void function(String f) {
		FunctionExpression<T> funct = new FunctionExpression<T>(f,math);
		Expression<T> expr = expressions.getLast();
		expr.addChild(funct);
		expressions.add(funct);
	}
	@Override
	void number(String val) throws Exception {
		T t;
		try {
			t = math.util.parse(val);
			t = math.util.copy(t,math.PRECISION+100);
		} catch(Exception e) {
			throw new Exception("Can't parse number "+val+"\n.");
		}
		ValueExpression<T> value = new ValueExpression<T>(t);
		Expression<T> expr = expressions.getLast();
		expr.addChild(value);
	}
	@Override
	void variable(String v) {
		ValueExpression<T> value = new ValueExpression<T>(v);
		Expression<T> expr = expressions.getLast();
		expr.addChild(value);
	}
}