/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package pattern;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import math.VecMath;

import common.Globals;
import common.Log;

public class CalculationRecords<T extends Number> {

	private String patternName;
	private List<Record> records;
	private VecMath<T> math;
	private short gridSize;
	public CalculationRecords(String name,short gridSize,VecMath<T> mt) {
		this.patternName = name;
		this.math = mt;
		this.gridSize = gridSize;
	}
	public CalculationRecords(String name,String content,VecMath<T> mt) 
	throws Exception {
		this(name,(short)-1,mt);
		StringTokenizer st = new StringTokenizer(content,"\n");
		int l = 0;
		short maxp = Globals.PRECISIONS[Globals.PRECISIONS.length-1];
		try {
			CalculationRecords<T>.Record cr = null;
			while(st.hasMoreTokens()) {
				if(cr==null)
					cr = new Record();
				l++;
				String line = st.nextToken().trim();
				int idx = line.indexOf("=");
				if(idx!=-1) {
					String key = line.substring(0,idx).trim();
					String val = line.substring(idx+1).trim();
					if(key.compareTo("radius")==0) {
						int k = val.indexOf("%");
						if(k!=-1)
							val = val.substring(0,k).trim();
						T value = math.util.copy(math.util.parse(val),maxp);
						cr.setShapeSize(value);
					} else if(key.equalsIgnoreCase("height")) {
						T value = math.util.copy(math.util.parse(val),maxp);
						cr.setHeight(value);
					} else if(key.equalsIgnoreCase("distance")) {
						T value = math.util.copy(math.util.parse(val),maxp);
						cr.setDistance(value);
					} else if(key.equalsIgnoreCase("angle")) {
						T value = math.util.copy(math.util.parse(val),maxp);
						cr.setAngle(value);
					} else if(key.equalsIgnoreCase("size")) {
						int k = val.indexOf("x");
						if(k!=-1)
							val = val.substring(0,k).trim();
						short s = Short.parseShort(val);
						setGridSize(s);
					} else {
						throw new Exception("Unknown key: "+key);
					}
				}
				if(cr.getHeight()!=null&&cr.getDistance()!=null
						&&cr.getAngle()!=null) {
					add(cr);
					cr = null;
				}
			}
		} catch(Exception e) {
			Log.getIstc().logln("Error while parsing record "+name+".");
			Log.getIstc().logln("Can't read line "+l+".");
			throw e;
		}	
	}
	public void add(Record cr) {
		if(records==null)
			records = new ArrayList<Record>();
		if(getRecord(cr.shapeSize)==null) {
			int i = records.size();
			for(i = 0;i<records.size();i++)
				if(math.cmp(records.get(i).shapeSize,cr.shapeSize)>0)
					break;
			records.add(i,cr);
		}
	}
	public void add(T size,T distance,T height,T angle) {
		add(new Record(size,distance,height,angle));
	}
	public Record getRecord(int idx) {
		return records.get(idx);
	}
	public Record getRecord(T size) {
		for(Record cr:records)
			if(math.cmp(size,cr.shapeSize)==0)
				return cr;
		return null;
	}
	public List<Record> getRecords() {
		return new ArrayList<Record>(records);
	}
	public void setRecords(List<Record> recs) {
		records = recs;
	}
	public short getGridSize() {
		return gridSize;
	}
	public String getPatternName() {
		return patternName;
	}
	public void setPatternName(String name) {
		this.patternName = name;
	}
	public void setGridSize(short s) {
		this.gridSize = s;
	}
	public String toString() {
		StringBuffer sb = new StringBuffer("size     = ");
		sb.append(gridSize).append(" x ").append(gridSize);
		sb.append("\n");
		if(records.size()>1)
			sb.append("\n");
		for(Record cr:records)
			sb.append(cr).append("\n");
		return sb.toString();	
	}

	public class Record {

		private T shapeSize;
		private T height;
		private T angle;
		private T distance;
		public Record() {}
		public Record(T s,T d,T h,T phi) {
			this.shapeSize = s;
			this.height = h;
			this.distance = d;
			this.angle = phi;
		}
		public T getShapeSize() {
			return shapeSize;
		}
		public T getHeight() {
			return height;
		}
		public T getAngle() {
			return angle;
		}
		public T getDistance() {
			return distance;
		}
		public void setShapeSize(T s) {
			this.shapeSize = s;
		}
		public void setHeight(T h) {
			this.height = h;
		}
		public void setAngle(T a) {
			this.angle = a;
		}
		public void setDistance(T d) {
			this.distance = d;
		}
		public CalculationRecords<T> getEnclosingType(){
			return CalculationRecords.this;
		}
		public String toString() {
			StringBuffer sb = new StringBuffer();
			sb.append("radius   = ").append(shapeSize);
			sb.append("%\n");
			sb.append("height   = ").append(height);
			sb.append("\n");
			sb.append("distance = ").append(distance);
			sb.append("\n");
			sb.append("angle    = ").append(angle);
			sb.append("\n");
			return sb.toString();		
		}
	}
}
