/**
	Copyright (c) 2009-2013 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package pattern;
import java.util.ArrayList;
import java.util.List;

public class LightPatternSequence implements ILightPatternSequence {

	protected List<short[]> cells;
	protected List<Float> m_init;
	LightPatternSequence() {}
	LightPatternSequence(LightPatternSequence lps) {
		cells = new ArrayList<short[]>();
		for(short[] cell:lps.getCellSequence())
			cells.add(cell.clone());
		m_init = new ArrayList<Float>();
		for(float x:lps.getInitialGradients())
			m_init.add(x);
	}
	@Override
	public boolean equals(Object o) {
		LightPatternSequence s = (LightPatternSequence)o;
		boolean b = true;
		b &= (cells!=null)==(s.cells!=null);
		b &= (m_init!=null)==(s.m_init!=null);
		if(cells!=null&&m_init!=null) {
			b &= cells.size()==s.cells.size();
			b &= m_init.size()==s.m_init.size();
			if(b) {
				for(int i = 0;i<cells.size();i++) {
					b &= cells.get(i)[0]==s.cells.get(i)[0];
					b &= cells.get(i)[1]==s.cells.get(i)[1];
				}
				for(int i = 0;i<m_init.size();i++)
					b &= m_init.get(i).floatValue()==s.m_init.get(i).floatValue();
			}
		}
		return b;
	}
	public void setCellSequence(List<short[]> cells) {
		this.cells = cells;
	}
	public void setInitialGradients(List<Float> mInit) {
		this.m_init = mInit;
	}
	public List<short[]> getCellSequence() {
		return cells;
	}
	public List<Float> getInitialGradients() {
		return m_init;
	}
}
