/**
	Copyright (c) 2009-2012 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package test;
import junit.framework.Assert;
import junit.framework.TestCase;
import lattice.Circle;
import lattice.Ray;
import math.VecMath;
import math.VecMathApfloat;
import math.VecMathFloat;

import org.apfloat.Apfloat;
import org.junit.Test;

public class CircleTest {

	private static final double DELTA = 0.000001;
	@Test
	public void test2DReflect() {
		VecMathFloat math = new VecMathFloat();
		Float[] center = new Float[]{1.0f,1.0f};
		Circle<Float> sphere = new Circle<Float>(center,1.0f,math);
		Float[] start = new Float[]{3.0f,-1.0f};
		Float[] dir = new Float[]{-1.0f,1.0f};
		math.normalize(dir);
		Ray<Float> ray1 = new Ray<Float>(start,dir,null);
		Ray<Float> ray2 = new Ray<Float>(new Float[2],new Float[2],null);
		Float t = (float)Math.sqrt(0.5f);
		Float[] res1 = new Float[]{1+t,1-t};
		Float[] res2 = new Float[]{1.0f,-1.0f};
		math.normalize(res2);
		sphere.reflect(ray1,ray2);
		TestCase.assertEquals("Start x",res1[0],ray2.getStart()[0],DELTA);
		TestCase.assertEquals("Start y",res1[1],ray2.getStart()[1],DELTA);
		TestCase.assertEquals("dir x",res2[0],ray2.getDir()[0],DELTA);
		TestCase.assertEquals("dir y",res2[1],ray2.getDir()[1],DELTA);
	}
	@Test
	public void testPrecise2DReflect() {
		VecMathApfloat math = new VecMathApfloat((short)100);
		Apfloat[] center = new Apfloat[]{math.ZERO,math.ZERO};
		Circle<Apfloat> sphere = new Circle<Apfloat>(center,math.ONE,math);
		Apfloat t = math.sqrt(math.util.copy(math.ONE_OVER_TWO,math.PRECISION));
		Apfloat[] start = new Apfloat[]{math.util.cast(-2),t};
		Apfloat[] dir = new Apfloat[]{math.util.cast(1),math.ZERO};
		Apfloat[] resStart = new Apfloat[]{math.neg(t),t};
		Apfloat[] resDir = new Apfloat[]{math.ZERO,math.util.cast(1)};
		Ray<Apfloat> ray1 = new Ray<Apfloat>(start,dir,null);
		Ray<Apfloat> ray2 = new Ray<Apfloat>(new Apfloat[2],new Apfloat[2],null);
		sphere.reflect(ray1,ray2);
		Apfloat epsilon = math.mul(math.EPSILON,math.util.cast(100));
		assertEquals("Start x",resStart[0],ray2.getStart()[0],epsilon,math);
		assertEquals("Start y",resStart[1],ray2.getStart()[1],epsilon,math);
		assertEquals("dir x",resDir[0],ray2.getDir()[0],epsilon,math);
		assertEquals("dir y",resDir[1],ray2.getDir()[1],epsilon,math);
		start = new Apfloat[]{math.util.cast(2),math.neg(t)};
		dir = new Apfloat[]{math.util.cast(-1),math.ZERO};
		resStart = new Apfloat[]{t,math.neg(t)};
		resDir = new Apfloat[]{math.ZERO,math.util.cast(-1)};
		ray1 = new Ray<Apfloat>(start,dir,null);
		ray2 = new Ray<Apfloat>(new Apfloat[2],new Apfloat[2],null);
		sphere.reflect(ray1,ray2);
		assertEquals("Start x",resStart[0],ray2.getStart()[0],epsilon,math);
		assertEquals("Start y",resStart[1],ray2.getStart()[1],epsilon,math);
		assertEquals("dir x",resDir[0],ray2.getDir()[0],epsilon,math);
		assertEquals("dir y",resDir[1],ray2.getDir()[1],epsilon,math);
	}
	private <T extends Number>void assertEquals(String msg,T val,T res,T diff,VecMath<T> m) {
		if(m.cmp(m.abs(m.sub(val,res)),diff)>0) {
			System.out.println("Test failed");
			System.out.println(msg);
			System.out.println("Expected: "+val);
			System.out.println("Got: "+res);
			System.out.println("Difference: "+m.sub(val,res));
			System.out.println("Max Difference: "+diff);
			Assert.assertEquals(msg,true,false);
			new Exception("Error location: ").printStackTrace();
			System.exit(0);
		}
	}
}
