/**
	Copyright (c) 2009-2012 by Zoltan Palmer

	This file is part of FroZenLight.

    FroZenLight is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FroZenLight is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FroZenLight.  If not, see http://www.gnu.org/licenses/
*/
package test;
import math.VecMath;
import math.VecMathApfloat;

import org.apfloat.Apfloat;
import org.junit.Test;

import parser.ExpressionTree;
import parser.ExpressionTreeBuilder;

public class ParserTest {

	@Test
	public void testParser() {
		VecMathApfloat mt = new VecMathApfloat((short)100);
		Apfloat epsilon = mt.mul(mt.EPSILON,mt.util.cast(100));
		ExpressionTreeBuilder<Apfloat> exprBd = new ExpressionTreeBuilder<Apfloat>(mt);
		try {
			String mathExpr = "0.8E-2";
			Apfloat result = mt.util.parse("0.008");
			ExpressionTree<Apfloat> exprTree = exprBd.build(mathExpr);
			Apfloat val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "2*0.55/5*10";
			result = mt.util.parse("2.2");
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "-3^2";
			result = mt.util.cast(-9);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "-3-(-2-(-4-(-3-9)))";
			result = mt.util.cast(7);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "8+2*9-4^2";
			result = mt.util.cast(10);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "1/3+2!";
			result = mt.div(mt.util.cast(7),mt.util.copy(mt.util.cast(3),mt.PRECISION));
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "1/3-3^(-1)";
			result = mt.util.cast(0);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "2*(4^2+3^2)/(7-2)^2";
			result = mt.util.cast(2);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "sqrt(4^2+3^2)";
			result = mt.util.cast(5);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "sin(1)^2+cos(1)^2";
			result = mt.util.cast(1);
			exprTree = exprBd.build(mathExpr);
			val = exprTree.calculate("",null);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			mathExpr = "sin(x)";
			result = mt.util.cast(0);
			exprTree = exprBd.build(mathExpr);
			Apfloat[] arr = new Apfloat[]{mt.PI};
			val = exprTree.calculate("x=$1",arr);
			val = mt.util.copy(val,mt.PRECISION);
			assertEquals("Result",result,val,epsilon,mt);
			String mathExpr1 = "sin(2*x)";
			String mathExpr2 = "2*sin(x)*cos(x)";
			arr = new Apfloat[]{mt.ONE};
			Apfloat y1 = exprBd.build(mathExpr1).calculate("x=$1",arr);
			Apfloat y2 = exprBd.build(mathExpr2).calculate("x=$1",arr);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			assertEquals("Test formula 1",mt.ZERO,mt.sub(y1,y2),epsilon,mt);
			mathExpr1 = "sin(4*x)";
			mathExpr2 = "cos(x)*[4*sin(x)-8*sin(x)^3]";
			arr = new Apfloat[]{mt.ONE};
			y1 = exprBd.build(mathExpr1).calculate("x=$1",arr);
			y2 = exprBd.build(mathExpr2).calculate("x=$1",arr);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			assertEquals("Test formula 2",mt.ZERO,mt.sub(y1,y2),epsilon,mt);
			mathExpr1 = "sin(x)";
			mathExpr2 = "sqrt((-cos(2*x)+1)/2)";
			arr = new Apfloat[]{mt.ONE};
			y1 = exprBd.build(mathExpr1).calculate("x=$1",arr);
			y2 = exprBd.build(mathExpr2).calculate("x=$1",arr);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			assertEquals("Test formula 3",mt.ZERO,mt.sub(y1,y2),epsilon,mt);
			mathExpr1 = "tan(x-y)";
			mathExpr2 = "(tan(x)-tan(y))/(1+tan(x)*tan(y))";
			arr = new Apfloat[]{mt.ONE,mt.TWO};
			y1 = exprBd.build(mathExpr1).calculate("x=$1,y=$2",arr);
			y2 = exprBd.build(mathExpr2).calculate("x=$1,y=$2",arr);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			assertEquals("Test formula 4",mt.ZERO,mt.sub(y1,y2),epsilon,mt);
			mathExpr1 = "(x-x^3/3!+x^5/5!-x^7/7!+x^9/9!-x^11/11!+x^13/13!)";
			mathExpr1 = mathExpr1+"/(1-x^2/2!+x^4/4!-x^6/6!+x^8/8!-x^10/10!+x^12/12!)";
			mathExpr2 = "tan(x)";
			y1 = exprBd.build(mathExpr1).calculate("x=0.1",null);
			y2 = exprBd.build(mathExpr2).calculate("x=0.1",null);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			Apfloat eps = new Apfloat(1E-25);
			assertEquals("Test formula 5",mt.ZERO,mt.sub(y1,y2),eps,mt);
			mathExpr1 = "sin(x^2)";
			mathExpr2 = "sin(sin(arcsin(x))^2)";
			arr = new Apfloat[]{new Apfloat("0.712")};
			y1 = exprBd.build(mathExpr1).calculate("x=$1",arr);
			y2 = exprBd.build(mathExpr2).calculate("x=$1",arr);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			assertEquals("Test formula 6",mt.ZERO,mt.sub(y1,y2),epsilon,mt);
			mathExpr1 = "x^y";
			mathExpr2 = "e^(y*ln(x))";
			arr = new Apfloat[]{mt.inv(mt.THREE)};
			y1 = exprBd.build(mathExpr1).calculate("x=0.21E-1,y=$1",arr);
			y2 = exprBd.build(mathExpr2).calculate("x=0.21E-1,y=$1",arr);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			assertEquals("Test formula 7",mt.ZERO,mt.sub(y1,y2),epsilon,mt);
			mathExpr1 = "1+1/(2+1/(2+1/(2+1/(2+1/(2+1/(2+1/(2+1/(2+1/(2+1/(2+1/2))))))))))";
			mathExpr2 = "sqrt(2)";
			y1 = exprBd.build(mathExpr1).calculate("",null);
			y2 = exprBd.build(mathExpr2).calculate("",null);
			y1 = mt.util.copy(y1,mt.PRECISION);
			y2 = mt.util.copy(y2,mt.PRECISION);
			eps = new Apfloat(1E-8);
			assertEquals("Test formula 8",mt.ZERO,mt.sub(y1,y2),eps,mt);
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	private <T extends Number>void assertEquals(String msg,T val,T res,T diff,VecMath<T> m) {
		if(m.cmp(m.abs(m.sub(val,res)),diff)>0) {
			System.out.println("Test failed");
			System.out.println(msg);
			System.out.println("Expected: "+val);
			System.out.println("Got: "+res);
			System.out.println("Difference: "+m.sub(val,res));
			System.out.println("Max Difference: "+diff);
			Assert.assertEquals(msg,true,false);
			new Exception("Error location: ").printStackTrace();
			System.exit(0);
		}
	}
}
